


local storage = minetest.get_mod_storage()

local mod_name = minetest.get_current_modname()
local mod_dir  = minetest.get_modpath( mod_name ) .. "/"
local data_dir = mod_dir .. "data/"
local main_marsbase_created = storage:get_int("main_marsbase_created")
local main_marsbase_counter = 10


local file_marsbase = data_dir .. "marsbase.we" -- size(x, y, z): 31, 11, 33


local function load_we(pos, filename)
    print("loading worldedit file:", filename, "to point", dump(pos))
    local file, err = io.open(filename, "rb")
    if err then
        print("cannot open file:", filename)
        return
    end
    local value = file:read("*a")
    file:close()
    worldedit.deserialize(pos, value)
end


local function build_marsbase(pos)
    print("build marsbase at point:", dump(pos))
    local p = {x = pos.x - 16, y = pos.y - 2, z = pos.z - 17}
    load_we(p, file_marsbase)
end


local function flying_isle(pos, radius, noise)
    print("flying isle at:", dump(pos))

    local maxRadius = radius + noise
    local pos1 = { x = pos.x - maxRadius , y = pos.y - maxRadius , z = pos.z - maxRadius }
    local pos2 = { x = pos.x + maxRadius , y = pos.y + maxRadius , z = pos.z + maxRadius }

    local vm = minetest.get_voxel_manip()
    local emin, emax = vm:read_from_map(pos1, pos2);
    local area = VoxelArea:new{MinEdge = emin, MaxEdge = emax}
    local data = vm:get_data()
    local c_air   = minetest.get_content_id("air")
    local c_stone = minetest.get_content_id("default:stone")
    local c_sand  = minetest.get_content_id("default:sand")

    for dz = -maxRadius, maxRadius do
    for dy = -maxRadius, maxRadius do
    for dx = -maxRadius, maxRadius do
        local r = math.random(radius, maxRadius)
        if dx*dx + dy*dy + dz*dz <= r*r then
            local i = area:index(pos.x + dx, pos.y + dy, pos.z + dz)
            if dy > 0 then
                data[i] = c_air
            elseif dy == 0 then
                data[i] = c_sand
            elseif dy == -1 then
                data[i] = c_stone
            elseif data[i] == c_air then
                data[i] = c_stone
            end
        end
    end end end

    vm:set_data(data)
    vm:write_to_map(true)
end


local function place_marsbase(pos)
    flying_isle(pos, 20, 1)
    build_marsbase(pos)
end


local function create_main_marsbase()
    print("create main marsbase")
    for i = 1, 100 do
        local d = 100 + i*10
        local x = math.random(-100, 100)
        local z = math.random(-100, 100)
        local y = minetest.get_spawn_level(x, z)
        if y then
            place_marsbase({x = x, y = y, z = z})
            return true
        end
    end
    print("cannot find a place for the main marsbase")
    return false
end


if main_marsbase_created ~= 1 then
    minetest.register_globalstep(function(dt)
        if main_marsbase_created == 1 then return end
        main_marsbase_counter = main_marsbase_counter - dt
        if main_marsbase_counter > 0 then return end
        if math.random(1, 100) > 10 then return end
        if create_main_marsbase() then
            main_marsbase_created = 1
            storage:set_int("main_marsbase_created", 1)
        end
    end)
end



minetest.register_tool("mars_base:magictool", {
    description = "Marsbase magic tool",
    inventory_image = "magictool.png",
    groups = {tool = 1},
    on_use = function(itemstack, user, pointed_thing)
        if pointed_thing.type == "node" then
            build_marsbase(pointed_thing.under)
        end
    end,
    on_place = function(itemstack, user, pointed_thing)
        if pointed_thing.type == "node" then
            place_marsbase(pointed_thing.under)
        end
    end,
})


